#libraries for classification
library(MLmetrics)   # caret requirement
library(MLeval)      # plot ROC curves from caret
library(gbm)         # caret requirement for Stochastic Gradient Boosting models
library(ranger)      # caret requirement for random forest models
library(rattle)      # visualizing decision trees
library(caret)       # interface to ML algorithms
library(pROC)
library(C50)         # contains the churn dataset
library(datasets)
library(modeldata)
library(factoextra) 
library(cluster)            ## visualize clustering output

set.seed(12L)
head(iris)
summary(iris)

iris_classification <- droplevels(iris[iris$Species!="versicolor",]) #drop versicolor for training
rownames(iris_classification) <- 1:nrow(iris_classification)

table(iris$Species)
table(iris_classification$Species)


# boxplot for each attribute on one image
x <- iris_classification[,1:4]
y <- iris_classification[,5]
par(mfrow=c(1,4))
for(i in 1:4) {
  boxplot(x[,i], main=names(iris_classification)[i])
}
dev.off()

# scatterplot matrix
featurePlot(x=x, y=y, plot="ellipse")
# box and whisker plots for each attribute
featurePlot(x=x, y=y, plot="box")

# do a 3:4 train test split
index <- createDataPartition(iris_classification$Species,p=.75, list=F)

train.set = iris_classification[index,]
test.set = iris_classification[-index,]


#k-fold crossvalidation
fitControl <- trainControl(
  method = "repeatedcv",
  # K=10 in K-fold cross-validation
  number = 10,
  # repeated ten times
  repeats = 10,
  classProbs = T,
  savePredictions = T)


?caret::train
# fit a decision tree model
iris.tree = train(Species ~ ., 
                  data=train.set, 
                  method="rpart", #CART algorithm
                  trControl = fitControl) # using cross validation
iris.tree
summary(iris.tree$finalModel)

plot(iris.tree$finalModel, uniform=TRUE,
     main="Classification Tree")
text(iris.tree$finalModel, cex=.8)

fancyRpartPlot(iris.tree$finalModel)

# estimate model quality
iris.pred = predict(iris.tree, newdata = test.set)
table(iris.pred, test.set$Species)

error.rate = round(mean(iris.pred != test.set$Species),2)
error.rate

confusionMatrix(iris.pred, test.set$Species)
print(iris.pred)


#fit a Gradient Boosting Algorithm
iris.gbm = train(Species ~ ., 
                 data=train.set, 
                 method="gbm", 
                 trControl = fitControl,
                 # preProcess = c("center","scale"),
                 verbose=F) 
iris.gbm

# estimate model quality
iris.pred = predict(iris.gbm, newdata = test.set)
error.rate = round(mean(iris.pred != test.set$Species),2)
error.rate


# Validation with whole iris dataset

iris.pred2 = predict(iris.tree, newdata = iris)
table(iris.pred2, iris$Species)


confusionMatrix(iris.pred2, iris$Species)
print(iris.pred2)

#unsupervised learning
# k-means clustering - Check for best k
k <- 3
i <- grep("Length", names(iris))
x <- iris[,i]
km <- kmeans(x, k)
km
plot(x, col=km$cluster)
points(km$centers, col=c(1:k), pch=8, cex=1.6, lwd=1.5)

# How to find the optimal k?
ks <- 1:5
collect_within_ss <- sapply(ks, function(k){
  cl <- kmeans(x, k)
  cl$tot.withinss
})
plot(ks, collect_within_ss, type="b",
     ylab="Total within squared distances", xlab="Tested value of k")
silhouette(km$cluster, dist(x)) |> fviz_silhouette()
fviz_nbclust(x, kmeans, method = "silhouette", k.max = 10)
#
km <- kmeans(x, 2)
plot(x, col=km$cluster)
points(km$centers, col=c(1:2), pch=8, cex=1.6, lwd=1.5)

# hierarchical clustering
d <- dist(iris[,1:4])
hcl <- hclust(d)
hcl
par(mfrow = c(1, 1))
plot(hcl)
abline(h=3, col="red")
# grab cluster assignments for a value of k
# use either k=X or h=X
cutree(hcl, k=2)
cutree(hcl, h=3)
km$cluster
table(km$cluster, cutree(hcl, k=2))
par(mfrow = c(1, 2))
plot(iris$Petal.Length, iris$Sepal.Length, col = km$cluster, main = "k-means")
plot(iris$Petal.Length, iris$Sepal.Length, col = cutree(hcl, h = 3), main = "Hierarchical clustering")
dev.off()
par(mfrow = c(1, 3))
plot(iris$Petal.Length, iris$Sepal.Length, col = cutree(hcl, k = 2), main = "Hierarchical clustering")
plot(iris$Petal.Length, iris$Sepal.Length, col = cutree(hcl, k = 3), main = "Hierarchical clustering")
plot(iris$Petal.Length, iris$Sepal.Length, col = cutree(hcl, h = 3), main = "Hierarchical clustering")
dev.off()

table(iris$Species, km$cluster)
table(iris$Species, cutree(hcl,h=4))

# scaling
par(mfrow = c(1, 2))
mtcars |> dist() |> hclust() |> plot()
mtcars |> scale() |> dist() |> hclust() |> plot()
dev.off()



